# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

package FomaInteractive;
use strict;
use Carp 'croak';
use IPC::Run;

use constant default_foma_program => 'foma';

sub new {
  my $class = shift;
  my %self = (foma_program => $class->default_foma_program,
              to_str   => '',
              from_str => '',
              counter  => 0,
              verbose  => 0,
              @_);
  $self{'h'} = IPC::Run::start([$self{'foma_program'},
                                '-p',  # pipe mode
                                '-r',  # no readline
                               ],
                               '<', \$self{'to_str'},
                               '>', \$self{'from_str'});
  if (defined $self{'tee'}) {
    open my $fh, '>', $self{'tee'}
      or croak "Cannot create tee file: $!";
    print $fh "#!/usr/bin/foma -f\n" or die;
    close $fh or die;
  }
  return bless \%self, $class;
}

sub command {
  my $self = shift;
  my $ret = '';
  foreach my $str (@_) {
    if (defined $self->{'tee'}) {
      open my $fh, '>>', $self->{'tee'} or die;
      print $fh $str, "\n" or die;
      close $fh or die;
    }
    my $end = "FOMA-INTERACTIVE-END $self->{'counter'}\n";
    $self->{'to_str'} = $str . "\n";
    if ($self->{'verbose'}) {
      print $self->{'to_str'};
    }
    $self->{'to_str'} .= "echo $end\n";
    $self->{'from_str'} = '';
    do {
      $self->{'h'}->pump;
      if ($self->{'verbose'} >= 2) {
        print "at from_str \"$self->{'from_str'}\"\n";
      }
    } until ($self->{'from_str'} =~ /^(.*)\Q$end/s);
    $ret = $1;
    if ($self->{'verbose'}) {
      print "---\n$ret---\n\n";
    }
  }
  return $ret;
}

sub test {
  my ($self, $str) = @_;
  $str = $self->command("test $str");
  if ($str =~ /^([01]) \(1 = TRUE, 0 = FALSE\)/m) {
    return $1;
  } else {
    croak "Unrecognised foma output: $str";
  }
}
sub shortest_string_size {
  my ($self, $str) = @_;
  $str = $self->command('print shortest-string-size');
  if ($str =~ /^Shortest acyclic path length: (\d+)$/m) {
    return $1;
  } else {
    croak "Unrecognised foma output: $str";
  }
}

1;
__END__
